#include <p18f2550.h>
#include <delays.h>
#include "1wire.h"

//#define DQ_LOW			PORTAbits.RA4=0; TRISA = 0b00000000; 	// pin = low, port = output
//#define DQ_RELEASE		TRISA = 0b00010000;						// port = input (ie tristate)
//#define DQ_READ			PORTAbits.RA4;

#define DQ_LOW			LATAbits.LATA4=0; TRISAbits.TRISA4=0; 	// pin = low, port = output
#define DQ_RELEASE		TRISAbits.TRISA4=1;						// port = input (ie tristate)
#define DQ_READ			PORTAbits.RA4;

void cal(void)
{
		while (1)
		{
			DQ_LOW									// take DQ low
			Delay10TCYx(10);						// 50uSec.
			DQ_RELEASE; 							// release DQ, let 4k7 pullup take the bus high
			Delay10TCYx(10);						// 50uSec.
		}
}

void OWCalibrate(int timebase)
{
	switch (timebase)
	{
	case 5:											// generate 5uSec squarewave
		while (1)
		{
			DQ_LOW									// take DQ low
			Delay10TCYx(1);							// 5uSec.
			DQ_RELEASE; 							// release DQ, let 4k7 pullup take the bus high
			Delay10TCYx(1);							// 5uSec.
		}
		
	case 50:										// generate 50uSec squarewave
		while (1)
		{
			DQ_LOW									// take DQ low
			Delay10TCYx(20);						// 50uSec.
			DQ_RELEASE; 							// release DQ, let 4k7 pullup take the bus high
			Delay10TCYx(20);						// 50uSec.
		}
	}
	return;
}

//-----------------------------------------------------------------------------
// Generate a 1-Wire reset
//
int OWReset(void)
{
	int result=1;

	DQ_LOW
	Delay10TCYx(100);						// 500uSec. drive DQ low for minimum of 480 uSec
	DQ_RELEASE; 							// release DQ, let 4k7 pullup take the bus high
	Delay10TCYx(12);						// 60uSec. within 15 to 60uSec, DS1820 will pull bus low
	result = (int)DQ_READ
	Delay10TCYx(48); 						// 240usec. ensure 'presence' pulse is complete
	return (result);						// 0 = DS1820 found
}

//-----------------------------------------------------------------------------
// Send a 1-Wire write bit.
//
void OWWriteBit(int bit)
{
	if (bit)
	{
		// Write '1' bit
		DQ_LOW								// drive DQ low to generate a time slot
		Delay10TCYx(1);						// 5uSec.
		DQ_RELEASE							// release DQ. allow DQ to float high, generating a 'write 1' slot
		Delay10TCYx(12);					// 60usec. Wait for DS1220 to finish sampling the 1 bit
	}
	else
	{
		// Write '0' bit
		DQ_LOW								// drive DQ low to generate a time slot
		Delay10TCYx(12);					// 60uSec. hold bus low, generating a 'write 0' slot
		DQ_RELEASE							// release DQ
	}

	Delay10TCYx(2);							// 10uSec recovery time
}

//-----------------------------------------------------------------------------
// Read a bit from the 1-Wire bus and return it
//
int OWReadBit(void)
{
	int result=0;

	DQ_LOW										// drive DQ low 
	Delay10TCYx(1);								// 5uSec delay
	DQ_RELEASE									// release DQ to generate a 'Read' time slot
	Delay10TCYx(1);								// 5uSec. wait for read slot data to be valid
	result = (int)DQ_READ		 				// Sample the bus
	Delay10TCYx(12);							// 60uSec. complete the time slot
	
	return (result);
}

//-----------------------------------------------------------------------------
// Write 1-Wire data byte
//
void OWWriteByte(int data)
{
	int loop;
	
	// Loop to write each bit in the byte, LS-bit first
	for (loop = 0; loop < 8; loop++)
	{
		OWWriteBit(data & 0x01);
		
		// shift the data byte for the next bit
		data >>= 1;
	}
}

//-----------------------------------------------------------------------------
// Read 1-Wire data byte and return it
//
int OWReadByte(void)
{
	int loop, result=0;
	
	for (loop = 0; loop < 8; loop++)
	{
		// shift the result to get it ready for the next bit
		result >>= 1;
		
		// if result is one, then set MS bit
		if (OWReadBit())
			result |= 0x80;
	}
	return result;
}

//-----------------------------------------------------------------------------
// Write a 1-Wire data byte and return the sampled result.
//
int OWTouchByte(int data)
{
	int loop, result=0;
	
	for (loop = 0; loop < 8; loop++)
	{
		// shift the result to get it ready for the next bit
		result >>= 1;
		
		// If sending a '1' then read a bit else write a '0'
		if (data & 0x01)
		{
			if (OWReadBit())
				result |= 0x80;
		}
		else
			OWWriteBit(0);
			
		// shift the data byte for the next bit
		data >>= 1;
	}
	return result;
}

//-----------------------------------------------------------------------------
// Write a block of 1-Wire data bytes and return the sampled result in the same buffer.
//
void OWBlock(unsigned char *data, int data_len)
{
	int loop;
	
	for (loop = 0; loop < data_len; loop++)
	{
		data[loop] = OWTouchByte(data[loop]);
	}
}

//-----------------------------------------------------------------------------
// Instruct device to convert temperature.
//
void Convert_Temperature(void)
{
	OWReset();								// master resets bus
	OWWriteByte(0xCC);						// skip ROM
	OWWriteByte(0x44);						// convert temperature
	OWReset();								// master resets bus
}

//-----------------------------------------------------------------------------
// Obtain a celsius temperature value from a DS18S20 device.
//
float Read_Temperature_DS18S20(void)
{

	// DS18S20 temperature register format :
	// LS Byte:	b7		b6		b5		b4		b3		b2		b1		b0
	//        	2^6		2^5		2^4		2^3		2^2		2^1		2^0		2^-1
	//
	// MS Byte:	b15		b14		b13		b12		b11		b10		b9		b8
	//        	S		S		S		S		S		S		S		S

	unsigned char get[10];
	int k, temp;
	float ftemp;

	OWWriteByte(0xCC);						// skip ROM
	OWWriteByte(0xBE);						// read scratch pad

	for (k=0; k<9; k++)						// read 9 bytes from scratchpad
		get[k]=OWReadByte();

	temp = get[0];							// load all 8 bits of the LSB

	if (get[1] > 0x80)						// sign bit set, temp is negative
		temp = temp * -1;

	ftemp =  temp;
    return (ftemp/2);
}

//-----------------------------------------------------------------------------
// Obtain a celsius temperature value from a DS18B20 device.
//
float Read_Temperature_DS18B20(void)
{
	// DS18B20 / DS1822 temperature register format :
	// LS Byte:	b7		b6		b5		b4		b3		b2		b1		b0
	//        	2^3		2^2		2^1		2^0		2^-1	2^-2	2^-3	2^-4
	//
	// MS Byte:	b15		b14		b13		b12		b11		b10		b9		b8
	//        	S		S		S		S		S		2^6		2^5		2^4

	unsigned char get[10];
	int k;
	long temp;
	float ftemp;

	OWWriteByte(0xCC);						// skip ROM
	OWWriteByte(0xBE);						// read scratch pad

	for (k=0; k<9; k++)						// read 9 bytes from scratchpad
		get[k]=OWReadByte();

	temp = get[1] & 0x7;					// load 3 bits of the MSB, masking any sign bits
	temp = temp << 8;						// shift the MSB left 8 bits
	temp = (temp | get[0]);					// load all 8 bits of the LSB

	if (get[1] > 0x80)						// sign bit set, temp is negative
		temp = temp * -1;

	ftemp = temp;
    return (ftemp/16);
}
