#include <p18cxxx.h>
#include <i2c.h>


/********************************************************************
*     Function Name:    EERandomRead                                *
*     Return Value:     error condition status and/or data byte     *
*     Parameters:       EE memory control byte with R/W set to 1    *
*     Description:      Reads 1 byte from passed address to EE memory*
*                       device. This routine can be used for any I2C*
*                       EE memory device, which only uses 1 byte of *
*                       address data as in the 24LC01B/02B/04B/08B. *
*                                                                   *  
********************************************************************/
unsigned int EByteRead( unsigned char control,unsigned char addressH, unsigned char address )
{
  	IdleI2C();                      // ensure module is idle
  	StartI2C();                     // initiate START condition
  	while ( SSPCON2bits.SEN );      // wait until start condition is over 
  	WriteI2C( control );    // write 1 byte
  	IdleI2C();                    // ensure module is idle
  	WriteI2C( addressH );
  	IdleI2C();                    // ensure module is idle
	WriteI2C( address );
    IdleI2C();                  // ensure module is idle
    RestartI2C();             // generate I2C bus restart condition
    while ( SSPCON2bits.RSEN );// wait until re-start condition is over 
    WriteI2C( control+1 );// write 1 byte - R/W bit should be 1
    IdleI2C();                // ensure module is idle
        if ( !SSPCON2bits.ACKSTAT )// test for ACK condition, if received
        {
          SSPCON2bits.RCEN = 1;       // enable master for 1 byte reception
          while ( SSPCON2bits.RCEN ); // check that receive sequence is over
          NotAckI2C();              // send ACK condition
          while ( SSPCON2bits.ACKEN ); // wait until ACK sequence is over 
          StopI2C();              // send STOP condition
          while ( SSPCON2bits.PEN ); // wait until stop condition is over 
    
        }
        else return ( -2 );          // return with Not Ack error
  	return ( (unsigned int) SSPBUF );     // return with data
}



/********************************************************************
*     Function Name:    EEByteWrite                                 *
*     Return Value:     error condition status                      *
*     Parameters:       EE memory device control, address and data  *
*                       bytes.                                      *
*     Description:      Write single data byte to I2C EE memory     *
*                       device. This routine can be used for any I2C*
*                       EE memory device, which only uses 1 byte of *
*                       address data as in the 24LC01B/02B/04B/08B. *
*                                                                   *  
********************************************************************/
unsigned char EByteWrite( unsigned char control, unsigned char addressH, unsigned char address, unsigned char data )
{
  	IdleI2C();                      // ensure module is idle
  	StartI2C();                     // initiate START condition
  	while ( SSPCON2bits.SEN );      // wait until start condition is over 
	WriteI2C( control );
    IdleI2C();                    // ensure module is idle
	WriteI2C( addressH );
    IdleI2C();                    // ensure module is idle
	WriteI2C( address );
    IdleI2C();                  // ensure module is idle
	WriteI2C( data );
  	IdleI2C();                      // ensure module is idle  
  	StopI2C();                      // send STOP condition
  	while ( SSPCON2bits.PEN );      // wait until stop condition is over 
    return ( 0 );                   // return with no error
}


/********************************************************************
*     Function Name:    EESequentialRead                            *
*     Return Value:     error condition status                      *
*     Parameters:       EE memory control, address, pointer and     *
*                       length bytes.                               *
*     Description:      Reads data string from I2C EE memory        *
*                       device. This routine can be used for any I2C*
*                       EE memory device, which only uses 1 byte of *
*                       address data as in the 24LC01B/02B/04B/08B. *
*                                                                   *  
********************************************************************/
unsigned char ESeqRead( unsigned char control, unsigned char addressH, unsigned char address, unsigned char *rdptr, unsigned char length )
{
  	IdleI2C();                      // ensure module is idle
  	StartI2C();                     // initiate START condition
  	while ( SSPCON2bits.SEN );      // wait until start condition is over 
	WriteI2C( control );
    IdleI2C();                    // ensure module is idle
	WriteI2C( addressH );
    IdleI2C();                    // ensure module is idle
	WriteI2C( address );
    IdleI2C();                  // ensure module is idle
	RestartI2C();             // generate I2C bus restart condition
    while ( SSPCON2bits.RSEN );  // wait until re-start condition is over 
    WriteI2C( control +1 );
    IdleI2C();                // ensure module is idle
        if ( !SSPCON2bits.ACKSTAT )// test for ACK condition received
        {
        getsI2C( rdptr, length );
        NotAckI2C();            // send not ACK condition
        while ( SSPCON2bits.ACKEN );// wait until ACK sequence is over 
        StopI2C();              // send STOP condition
        while ( SSPCON2bits.PEN );// wait until stop condition is over 
        }

  return ( 0 );                   // return with no error
}







/* $Id: i2c_puts.c,v 1.2 2005/04/11 21:22:35 nairnj Exp $ */
#include <p18cxxx.h>
#include <i2c.h>


signed char putslI2C( unsigned char *wrptr, unsigned char length )
{
  while ( length )                // transmit data until null character 
  {
    if ( SSPCON1bits.SSPM3 )      // if Master transmitter then execute the following
    {
      if ( putcI2C ( *wrptr ) )   // write 1 byte
      {
        return ( -3 );            // return with write collision error
      }
		length--;
      IdleI2C();                  // test for idle condition
      if ( SSPCON2bits.ACKSTAT )  // test received ack bit state
      {
        return ( -2 );            // bus device responded with  NOT ACK
      }                           // terminate putsI2C() function
    }

    else                          // else Slave transmitter
    {
      PIR1bits.SSPIF = 0;         // reset SSPIF bit
      SSPBUF = *wrptr;            // load SSPBUF with new data
      SSPCON1bits.CKP = 1;        // release clock line 
      while ( !PIR1bits.SSPIF );  // wait until ninth clock pulse received

      if ( ( !SSPSTATbits.R_W ) && ( !SSPSTATbits.BF ) )// if R/W=0 and BF=0, NOT ACK was received
      {
        return ( -2 );            // terminate PutsI2C() function
      }
    }

  wrptr ++;                       // increment pointer

  }                               // continue data writes until null character

  return ( 0 );
}



unsigned char EPageWrite( unsigned char control, unsigned char addressH,unsigned char address, unsigned char *wrptr, unsigned char len )
{
  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition
  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  WriteI2C( control );    // write 1 byte - R/W bit should be 0
  IdleI2C();                    // ensure module is idle
  WriteI2C( addressH);  // write address byte to EEPROM
  IdleI2C();                  // ensure module is idle
  WriteI2C( address);  // write address byte to EEPROM
  IdleI2C();                  // ensure module is idle
  putslI2C ( wrptr,len );
  IdleI2C();                      // ensure module is idle
  StopI2C();                      // send STOP condition
  while ( SSPCON2bits.PEN );      // wait until stop condition is over 
  return ( 0 );                   // return with no error
}
